<?php

/* ------------------------------------------------------------------------
  # NewebPay MPG Plugin
  # ------------------------------------------------------------------------
  # author    jack
  # version   1.0.1
  # MPG Version 1.4
  # copyright  COPYRIGHT 2018 © Neweb Technologies Co., Ltd. (https://www.newebpay.com/)
  # Websites: https://www.newebpay.com/
  ------------------------------------------------------------------------- */
  
class lib_common_newebpay
{
    private $MerchantId;//商店代號
    private $OrderNo;//訂單編號
    private $OrderAmt;//訂單總金額
    private $OrderItemName;//訂單物品名稱
    private $Email;
    private $ClientBackURL;
    private $ReturnURL;
    private $NotifyURL;
    private $LoginType;//是否要登入
    private $NewebPaytestMode;//測試模式
    private $HashKey;
    private $HashIv;
    private $Result;//解密結果
    private $LocalOrderPrice;//本地端訂單價格
    private $PaymentStatus = true; //訂單狀態

    function __call($method, $args)
    {
        $action = substr($method, 0, 3);
        $param = substr($method, 3);
        switch ($action) {
            case 'get':
                if (! isset($this->$param)) {
                    throw new exception($param.' is not set.');
                }

                return $this->$param;
            break;
            case 'set':
                if (! isset($args[0])) {
                    throw new exception($param.' param isnt set.');
                }

                $this->$param = $args[0];
                return $this;
            break;
            default:
                throw new exception('Method is not exisits.');
            break;
        }
    }

    function __construct()
    {
        
    }

    /**
     * 啟動MPG程式 輸出html
     *
     * @access public
     * @version 1.0.1
     * @return string
     */
    public function run()
    {
        return $this->generateMPGetwayData()->OutputFormHtml();
    }

    /**
     * 啟動MPG程式 輸出array
     *
     * @access public
     * @version 1.0.1
     * @return array
     */
    public function runOnlyFormData()
    {
        return $this->generateMPGetwayData()->OutputFormData();
    }

    /**
     * 啟動MPG解碼程式
     *
     * @access public
     * @param  array $return_data
     * @version 1.0.1
     * @return array
     */
    public function runReturnDataDecrypt($return_data)
    {
        if (empty($return_data)) {
            throw new exception("Return Data empty");
        }

        //比對TradeSha 是否正確
        if (! $this->chkShaIsVaildByReturnData($return_data)) {
            throw new exception("TradeSha : chk Error");
        }

        $result = $this->create_aes_decrypt($return_data['TradeInfo'],
            trim($this->getHashKey()),
            trim($this->getHashIv())
        );
        
        if (empty($result)) {
            throw new exception("AES decrypt : DataError");
        }
        $this->result = $result;
        return $result;
    }

    /**
     * 確認訂單回傳狀態
     *
     * @access public
     * @param  string $mode
     * @version 1.0.1
     * @return array
     */
    public function checkReturnOrderCondition($mode='notify')
    {
        switch ($mode) {
            case 'return':
                $this->checkOrderConditionIsVaild()
                     ->checkOrderAmtIsSame();
            break;
            case 'notify':
            default:
                $this->checkOrderConditionIsVaild()
                     ->checkOrderIsPaid()
                     ->checkOrderAmtIsSame();
            break;
        }
        return $this->orderOutputResult();
    }

    /**
     * 設定購物車版標籤(option)
     *
     * @access public
     * @param  string $value
     * @version 1.0.1
     * @return string
     */
    public function setCartVersion($value)
    {
        $this->CartVersion = (isset($value)) ? $value: '';
        return $this;
    }

    /**
     * 輸出Form HTML編碼
     *
     * @access private
     * @version 1.0.1
     * @return string
     */
    private function OutputFormHtml()
    {
        $this->postURL = ($this->getNewebPaytestMode() == '1') ? 'https://ccore.newebpay.com/MPG/mpg_gateway' : 'https://core.newebpay.com/MPG/mpg_gateway';
        $html ='<form id="newebpay" action="'.$this->postURL.'" method="post" name="adminForm" enctype="multipart/form-data">';

        foreach ($this->post_data as $key => $value) {
            $html .= '<input type="hidden" name="'.$key.'" value="'.$value.'">';
        }
        $html .='</form>';
        $html .='<script language=javascript>document.forms.newebpay.submit();</script>';

        return $html;
    }

    /**
     * 輸出Form Data
     *
     * @access private
     * @version 1.0.1
     * @return array|boolean
     */
    private function OutputFormData()
    {
        if (empty($this->post_data)) return false;
        return $this->post_data;
    }

    /**
     * 取得購物車版本標籤
     *
     * @access private
     * @param  string $value
     * @version 1.0.1
     * @return string
     */
    private function getCartVersion()
    {
        if (! empty($this->CartVersion)) return $this->CartVersion;
        return '';
    }

    private function getLangType()
    {
        if (empty($this->LangType)) {
            $this->LangType = 'zh-tw';
        }
        return $this->LangType;
    }

    /**
     * 產生MPG資料
     *
     * @access private
     * @param  string $value
     * @version 1.0.1
     * @return string
     */
    private function generateMPGetwayData() 
    {
        $hashkey = trim($this->getHashKey());
        $hashiv = trim($this->getHashIv());
        $pre_encrypt_data = array(
            'MerchantID' => trim($this->getMerchantId()),
            'RespondType' => 'JSON',
            'TimeStamp' => time(),
            'Version' => '1.4',
            'MerchantOrderNo' => $this->getOrderNo(),
            'Amt' => $this->getOrderAmt(),
            'ItemDesc' => $this->getOrderItemName(),
            'ClientBackURL' => $this->getClientBackURL(),
            'Email' => $this->getEmail(),
            'LoginType' => $this->getLoginType(),
            'ReturnURL' => $this->getReturnURL(),
            'NotifyURL' =>  $this->getNotifyURL(),
            'LangType' => $this->getLangType()
        );

        $aes = $this->create_mpg_aes_encrypt($pre_encrypt_data, $hashkey, $hashiv);
        $sha256 = $this->aes_sha256_str($aes, $hashkey, $hashiv);

        $this->post_data = array(
            'MerchantID' => $this->getMerchantId(),
            'TradeInfo' => $aes,
            'TradeSha' => $sha256,
            'Version' => '1.4',
            'Cart_version' => $this->getCartVersion()
        );

        return $this;
    }

    /**
     *MPG aes加密
     *
     * @access private
     * @param array $parameter ,string $key, string $iv
     * @version 1.0.1
     * @return string
     */
    private function create_mpg_aes_encrypt($parameter, $key = "", $iv = "")
    {
        $return_str = '';
        if (!empty($parameter)) {
            ksort($parameter);
            $return_str = http_build_query($parameter);
        }
        if (version_compare(PHP_VERSION, '5.6.0', '<') || ! function_exists('openssl_encrypt')) {
            return trim(bin2hex(@mcrypt_encrypt(MCRYPT_RIJNDAEL_128, $key,$this->addpadding($return_str),MCRYPT_MODE_CBC, $iv)));
        }

        return trim(bin2hex(openssl_encrypt($this->addpadding($return_str), 'aes-256-cbc', $key,OPENSSL_RAW_DATA|OPENSSL_ZERO_PADDING, $iv)));
    }

    private function addpadding($string, $blocksize = 32)
    {
        $len = strlen($string);
        $pad = $blocksize - ($len % $blocksize);
        $string .= str_repeat(chr($pad), $pad);
        return $string;
    }

     /**
     *MPG sha256加密
     *
     * @access private
     * @param string $str ,string $key, string $iv
     * @version 1.0.1
     * @return string
     */
    private function aes_sha256_str($str, $key = "", $iv = "")
    {
        return strtoupper(hash("sha256", 'HashKey='.$key.'&'.$str.'&HashIV='.$iv));
    }

    /**
     *MPG aes解密
     *
     * @access private
     * @param array $parameter ,string $key, string $iv
     * @version 1.0.1
     * @return array|boolean
     */
    private function create_aes_decrypt($parameter = "", $key = "", $iv = "")
    {
        if (version_compare(PHP_VERSION, '5.6.0', '<') || ! function_exists('openssl_decrypt')) {
            $decrypt_data = $this->strippadding(@mcrypt_decrypt(MCRYPT_RIJNDAEL_128, $key,
            hex2bin($parameter), MCRYPT_MODE_CBC, $iv));
        } else {
            $decrypt_data = $this->strippadding(openssl_decrypt(hex2bin($parameter),'AES-256-CBC', $key,OPENSSL_RAW_DATA|OPENSSL_ZERO_PADDING, $iv));
        }

        $decrypt_data = $this->strippadding(openssl_decrypt(hex2bin($parameter),'AES-256-CBC', $key,OPENSSL_RAW_DATA|OPENSSL_ZERO_PADDING, $iv));
        if(! $decrypt_data) return false;
        if(json_decode($decrypt_data)) {
            $return_data = $this->decrypt_json_data($decrypt_data);
        } else {
            $return_data = $this->decrypt_str_data($decrypt_data);
        }

        return $return_data;
    }

    private function strippadding($string) 
    {
        $slast = ord(substr($string, -1));
        $slastc = chr($slast);
        if (preg_match("/$slastc{" . $slast . "}/", $string)) {
            $string = substr($string, 0, strlen($string) - $slast);
            return $string;
        } else {
            return false;
        }
    }  

    private function decrypt_json_data($dec_str) 
    {
        $dec_data = json_decode($dec_str, true);
        $dec_data['Result']['Status'] = $dec_data['Status'];
        $dec_data['Result']['Message'] = $dec_data['Message'];
        return $dec_data['Result']; //整理成跟String回傳相同格式
    }

    private function decrypt_str_data($dec_str) 
    {
        $dec_data = explode('&', $dec_str);
        foreach ($dec_data as $_ind => $value) {
            $trans_data = explode('=', $value);
            $return_data[$trans_data[0]] = $trans_data[1];
        }
        return $return_data;
    }

    /**
     *檢查SHA256是否相符
     *
     * @access private
     * @param array $return_data 
     * @version 1.0.1
     * @return boolean
     */
    public function chkShaIsVaildByReturnData($return_data)
    {
        if (empty($return_data['TradeSha'])) return false;
        if (empty($return_data['TradeInfo'])) return false;
        $local_sha = $this->aes_sha256_str($return_data['TradeInfo'],
            trim($this->getHashKey()),
            trim($this->getHashIv())
        );
        if ($return_data['TradeSha'] != $local_sha) return false;
        return true;
    }

    /**
     *檢查訂單是否狀態是否成功
     *
     * @access private
     * @version 1.0.1
     * @return array
     */
    private function checkOrderConditionIsVaild()
    {
        if (empty($this->result)) {
            throw new Exception("Return Data Empty");
        }
        if (! $this->getPaymentStatus()) return $this;

        if (! in_array($this->result['Status'], array('SUCCESS', 'CUSTOM'))) {
            $this->setComment('錯誤訊息: '.urldecode($this->result['Message']));
            $this->setPaymentStatus(false);
        }
        return $this;
    }

    /**
     *檢查訂單是否付款
     *
     * @access private
     * @version 1.0.1
     * @return array
     */
    private function checkOrderIsPaid()
    {
        if (empty($this->result)) {
            throw new Exception("Return Data Empty");
        }
        if (! $this->getPaymentStatus()) return $this;
        
        if (empty($this->result['PayTime'])) {
            // $this->setComment($this->result['MerchantOrderNo'].'錯誤訊息: 訂單並未付款');
            // $this->setPaymentStatus(false);
            throw new Exception("Order is not paid.");
        }
        return $this;
    }

    /**
     *檢查訂單金額是否相符
     *
     * @access private
     * @version 1.0.1
     * @return array
     */
    private function checkOrderAmtIsSame()
    {   
        if (empty($this->result) || empty($this->getLocalOrderPrice())) {
            throw new Exception("Return Data or LocalOrderPrice Empty");
        }

        if (! $this->getPaymentStatus()) return $this;
        
        if ($this->getLocalOrderPrice() != $this->result['Amt']) {
            $this->setComment($this->result['MerchantOrderNo'].' 錯誤訊息: 付款訂單金額不一致');
            $this->setPaymentStatus(false);
        }
        return $this;
    }

    /**
     *取得訊息
     *
     * @access private
     * @version 1.0.1
     * @return string
     */
    private function getComment()
    {
        if ($this->getPaymentStatus()) {
            $this->Comment = $this->getSuccessComment();
        }
        return $this->Comment;
    }

    /**
     *輸出訂單檢查結果
     *
     * @access private
     * @version 1.0.1
     * @return array
     */
    private function orderOutputResult()
    {
        $order_cheeck_result = array(
            'comment' => $this->getComment(),
            'status' => $this->getPaymentStatus()
        );
        return $order_cheeck_result;
    }

    /**
     *取得訂單成功訊息
     *
     * @access private
     * @version 1.0.1
     * @return string
     */
    private function getSuccessComment()
    {
        if (empty($this->result)) {
            throw new Exception("Return Data Empty");
        }

        $pre = (isset($this->result['P2GPaymentType'])) ? 'ezPay電子錢包 - ' : '';

        switch ($this->result['PaymentType']) {
         case "CREDIT":
            $paymentNote = "付款方式:{$pre}信用卡(卡號末4碼:" . $this->result['Card4No'] . " 授權碼:" . $this->result['Auth'] . ")";
         break;
         case "WEBATM":
           $paymentNote = "付款方式:{$pre}WEBATM(匯款銀行:" . $this->result['PayBankCode'] . " 末五碼為:" . $this->result['PayerAccount5Code'] . ")";
         break;
         case "VACC":
           $paymentNote = "付款方式:{$pre}ATM轉帳(匯款銀行:" . $this->result['PayBankCode'] . " 末五碼為:" . $this->result['PayerAccount5Code'] . ")";
           break;
         case "CVS":
           $paymentNote = "付款方式:{$pre}超商代碼繳費";
           break;
         case "BARCODE":
           $paymentNote = "付款方式:{$pre}超商條碼繳費";
           break;
         case 'P2GEACC':
           $paymentNote = "付款方式:{$pre}電子帳戶";
           break;
         case 'ACCLINK':
           $paymentNote = "付款方式:{$pre}約定連結存款帳戶";
           break;
         default:
           $paymentNote = "付款方式:".$this->result['PaymentType'] . "(藍新金流交易序號:" . $this->result['TradeNo'] . ")";
           break;
      }
      return $paymentNote;
    }

    private function log_text_init(){
        $time = date("Y-m-d H:i:s");
        
        if(empty($_SERVER['HTTP_X_FORWARDED_FOR'])){
            $UserIP = $_SERVER['REMOTE_ADDR'];
        }else{
            $UserIP = explode(',', $_SERVER['HTTP_X_FORWARDED_FOR']);
            $UserIP = $UserIP[0];
        }
        
        if (!isset($_SERVER["HTTP_REFERER"])) {
            $referer = "(沒有來源頁面)";
        } else if ($_SERVER["HTTP_REFERER"] == "") {
            $referer = "(沒有來源頁面)";
        } else {
            $referer = $_SERVER["HTTP_REFERER"];
        }
        
        $text = "#" . $time;
        $text .= "||IP:" . $UserIP;
        $text .= "||來源:" . $referer;
        $text .= "||";
        return $text;
    }

    public function array_data_for_log($data) {
        if(is_array($data)){
            foreach($data as $index => $value){
                if(is_array($value)){
                    $text[] = $index  . ":" . json_encode($value);
                }else{
                    $text[] = $index  . ":" . $value;
                }
            }
            return implode('||', $text);
        } else {
            return $data;
        }
    }
    /**
     * 儲存交易回傳資料
     * 
     * @access private
     * @return void
     */
    public function write_log_in_file($dir, $log_text){
        if(!is_dir($dir)) {
            mkdir($dir);
        }
        $text = $this->log_text_init();
        $text .= $this->array_data_for_log($log_text) . "||POST:" . json_encode($_POST) . PHP_EOL;
        $fp = fopen($dir . 'newebpay_' . date("Ymd") . '.txt', "a+");
        fwrite($fp, $text);
        fclose($fp);
        $this->clean_old_log($dir);    //刪除1個月前的log
    }

    public function clean_old_log($dir){
        $del_date = date('Ymd', strtotime('-30 day'));
        $scan_dir = glob($dir . 'newebpay_*.txt');
        foreach ($scan_dir as $value) {
            $date = explode('_', basename($value, '.txt'));
            if (strtotime($del_date) > strtotime($date[1])) {
                unlink($value);
            }
        }
    }
}